#include <asm/uaccess.h>
#include <linux/errno.h>
#include <linux/mm.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/fs.h>
#include <linux/ioctl.h>
#include <linux/malloc.h>
#include <linux/locks.h>
#include <linux/smp.h>
#include <linux/smp_lock.h>
/* This is for kernel_thread */
#define __KERNEL_SYSCALLS__
#include <linux/unistd.h>

#include <linux/tcfs_fs.h>
#include <linux/tcfs/tcfs_hash.h>
#include <linux/tcfs/tcfs_library.h>

int tcfsusers;

int
tcfs_ioctl (struct inode *inode, struct file *filp, unsigned int cmd,
		 unsigned long arg)
{
	struct tcfs_user_key uk;
	struct tcfs_group_share gs; 
	int err=0;
	struct tcfs_header tcfsheader;
	struct tcfs_flags tcfsflags;
	char ciphername[TCFS_ENGINE_FIELD_SIZE];
	/*
	char tcfs_version[]={
                        TCFS_VERSION,
                        TCFS_VERSION_MINOR,
                        0,
			CIPHER_DES
                    }; 
	*/

	switch(cmd) {
/* ANIDEL: still to finish...
	case TCFS_IOC_SETUSERFL:
		copy_from_user ((char *)&tcfsuserfl, (char *)arg,
				sizeof(struct tcfs_user_flags));

		err=tcfs_put_userfl (filp, &tcfsuserfl);

		return err;

	case TCFS_IOC_GETUSERFL:
		copy_from_user ((char *)&tcfsuserfl, (char *)arg,
				sizeof(struct tcfs_user_flags));

		err=tcfs_get_userfl (filp, &tcfsuserfl);
		copy_to_user ((struct tcfs_user_flags *)arg,
				(char *)&tcfsuserfl,
				sizeof(struct tcfs_user_flags));
		return err;

*/
	case TCFS_IOC_GETFLAGS:
		tcfsheader.flags=0;
		err=tcfs_get_header (inode, &tcfsheader, NULL);
		/* If err is negative then the file is clear, if not
 		   tcfsheader.flags will just hold the correct flags */

		copy_to_user ((unsigned int *)arg, &tcfsheader.flags,
			sizeof(unsigned int));

		return 0;

	case TCFS_IOC_GETHEADER:
		err=tcfs_get_header (inode, &tcfsheader, NULL);
		if (err<0) {
			return -1;
		}

		copy_to_user ((struct tcfs_header *)arg, &tcfsheader,
				sizeof (struct tcfs_header));

		return 0;

	case TCFS_IOC_SETFLAGS:
		if (current->uid != inode->i_uid) {
			err=-EACCES;
			return err;
		}

		copy_from_user((char *)&tcfsflags, (char *)arg,
				sizeof(struct tcfs_flags));

		err = tcfs_modify_header (filp, NULL, TCFS_FIELD_FLAGS,
					(void *)&tcfsflags, NULL);

		if (err)
			return err;

		inode->u.tcfs_i.tcfsheader.flags=tcfsflags.flags;

		return -TCFS_ERROK;

	case TCFS_IOC_GETVERSION:
/*		put_user(*(long *)tcfs_version, (long *)arg);*/
		return 0;
	case TCFS_IOC_SETVERSION: /* for compatibility purpouses */
		return 0;

	/* Aggiungere la TCFS_IOC_GSETCIPHER per i gruppi */
	/* Forse non serve...se il file e' cifrato O e' X OPPURE e' G */
	case TCFS_IOC_SETCIPHER:
		memset (ciphername, 0, TCFS_ENGINE_FIELD_SIZE);
		copy_from_user (ciphername,(char *)arg,TCFS_ENGINE_FIELD_SIZE);
		return tcfs_modify_header (filp, NULL, TCFS_FIELD_ENGINE,
				ciphername, NULL);

	case TCFS_IOC_LOGIN:
		copy_from_user ((char *)&uk, (char *)arg,
			sizeof (struct tcfs_user_key));

		if (current->uid == uk.uid) {
			tcfs_add_userkey (&uk, 0);
			tcfsusers++;

			return 0;
		} else {
			return -EACCES;
		}

	case TCFS_IOC_LOGOUT:
		copy_from_user ((char *)&uk, (char *)arg,
			 sizeof(struct tcfs_user_key));

		if (current->uid == uk.uid) {
			tcfs_rem_userkey (&uk, 0);

			if (tcfsusers)
				tcfsusers--;

			return 0;
		} else {
			return -EACCES;
		}

	case TCFS_IOC_FULLOUT:
		copy_from_user ((char *)&uk, (char *)arg,
			 sizeof(struct tcfs_user_key));

		tcfs_rem_userkey (&uk, TCFS_ALLKEY_FLAG);
		if (tcfsusers)
			tcfsusers--;

		return 0;

	case TCFS_IOC_SETFIX:
		copy_from_user ((char *)&uk, (char *)arg,
			 sizeof(struct tcfs_user_key));

		tcfs_add_userkey (&uk, TCFS_PERMANENT_FLAG);

		return 0;

	case TCFS_IOC_GETFIX:
/*		copy_from_user ((char *)&uk, (char *)arg,
			 sizeof(struct tcfs_user_key));

		flags=tcfs_get_flags (uk);

		return flags;
*/
		return 0;

	case TCFS_IOC_GETCOUNT:
/*
		he=hash_lookup(current->uid);
		if (he==NULL) {
			return -EINVAL;
			put_user(0,(unsigned int *)arg);
		}
		put_user(he->count,(unsigned int *)arg);
*/
		return 0;

	case TCFS_IOC_DELFIX:
		copy_from_user ((char *)&uk, (char *)arg,
			 sizeof(struct tcfs_user_key));

		tcfs_rem_userkey (&uk, TCFS_PERMANENT_FLAG);

		return 0;

	case TCFS_IOC_GLOGIN:
		copy_from_user((char *)&gs,(char *)arg,
			sizeof(struct tcfs_group_share));

		tcfs_add_groupshare (&gs, 0);

		return 0;

	case TCFS_IOC_GLOGOUT:
		copy_from_user((char *)&gs,(char *)arg,
			sizeof(struct tcfs_group_share));

		tcfs_rem_groupshare (&gs, 0);

		return 0;

	case TCFS_IOC_GFULLOUT:
		copy_from_user((char *)&gs,(char *)arg,
			sizeof(struct tcfs_group_share));

		tcfs_rem_groupshare (&gs, TCFS_ALLKEY_FLAG);

		return 0;
	case TCFS_IOC_GSETFIX:
		copy_from_user((char *)&gs,(char *)arg,
			sizeof(struct tcfs_group_share));

		tcfs_add_groupshare (&gs, TCFS_PERMANENT_FLAG);

		return 0;

	case TCFS_IOC_GGETFIX:
/*
		copy_from_user(&gid,(unsigned int *)arg,sizeof(gid));
		if ((ghtmp=gid_hash_lookup(gid))==NULL)
			return -EINVAL;
		if ((he=gid_uid_lookup(ghtmp->key,current->uid))==NULL)
			return -EINVAL;
		put_user(he->permanent,(unsigned int *)arg);
*/
		return 0;

	case TCFS_IOC_GGETCOUNT:
/*
		copy_from_user(&gid,(unsigned int *)arg,sizeof(gid));
		if ((ghtmp=gid_hash_lookup(gid))==NULL)
			return -EINVAL;
		if ((he=gid_uid_lookup(ghtmp->key,current->uid))==NULL)
			return -EINVAL;
		put_user(he->count,(unsigned int *)arg);
*/
		return 0;

	case TCFS_IOC_GDELFIX:
		copy_from_user((char *)&gs,(char *)arg,
			sizeof(struct tcfs_group_share));

		tcfs_rem_groupshare (&gs, TCFS_PERMANENT_FLAG);

		return 0;
	default:
		return -ENOTTY;
	}
}
