#ifndef _TCFS_HASH_H
#define _TCFS_HASH_H

#include <linux/types.h>
#include <linux/tcfs.h>

#define TCFS_PERMANENT_FLAG	1
#define TCFS_ALLKEY_FLAG	2
#define HASH_SIZE		512

#define TCFS_IS_PERMANENT(flags)	(flags&TCFS_PERMANENT_FLAG)

struct tcfs_user_key {
	__kernel_uid_t uid;
        char engine[TCFS_ENGINE_FIELD_SIZE];
        unsigned char key[KEYSIZE];
};

struct tcfs_group_share {
        __kernel_uid_t uid;
        __kernel_gid_t gid;
        char engine[TCFS_ENGINE_FIELD_SIZE];
        unsigned int threshold;
        unsigned char share[GROUP_KEYSIZE];
};

struct user_key {
        __kernel_uid_t uid;
        char engine[TCFS_ENGINE_FIELD_SIZE];
        unsigned int count;
        unsigned char flags;
        void *key;
        struct user_key *next;
};

struct user_shares {
        __kernel_uid_t uid;
        unsigned int count;
        unsigned char flags;
        unsigned char *share;
        struct user_shares *next;
};

struct group_key {
        __kernel_gid_t gid;
        char engine[TCFS_ENGINE_FIELD_SIZE];
        unsigned int num_shares;
        struct user_shares *shares;
        void *key;
        struct group_key *next;
};

extern void tcfs_init_hash(void);
extern void tcfs_cleanup_hash(void);
extern struct user_key *tcfs_key_lookup (__kernel_uid_t, char *);
extern struct group_key *tcfs_groupkey_lookup (__kernel_gid_t, char *);
extern int tcfs_add_userkey (struct tcfs_user_key *, unsigned int);
extern int tcfs_rem_userkey (struct tcfs_user_key *, unsigned int);
extern int tcfs_add_groupshare (struct tcfs_group_share *, unsigned int);
extern int tcfs_rem_groupshare (struct tcfs_group_share *, unsigned int);

#endif
