#include <linux/kernel.h>
#include <linux/malloc.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/tcfs/tcfsapi.h>

#include "blowfish.h"
#define KEYSIZE 16

void *init_key (char *key)
{
	BLOWFISH_CTX *ctx;

	ctx=(BLOWFISH_CTX *)kmalloc (sizeof(BLOWFISH_CTX), GFP_KERNEL);

	blowfish_init (ctx, key, KEYSIZE);

	return ctx;
}

void encrypt(char *block, int nb, void *key)
{
	unsigned long *xi;
	unsigned long *xr, *xl;
	int i;
	char *tmp;
	BLOWFISH_CTX *ctx=(BLOWFISH_CTX *)key;

	xi=(long *)block;
	tmp=block;
	xl=(unsigned long *)tmp;
	xr=(unsigned long *)(tmp+4);
	blowfish_encrypt (ctx, xl, xr);
	tmp+=8;

	for (i=1;i<nb/8;i++) {
		*(xi+2)^=*xi;
		*(xi+3)^=*(xi+1);
		xl=(unsigned long *)tmp;
		xr=(unsigned long *)(tmp+4);
		blowfish_encrypt (ctx, xl, xr);
		tmp+=8;
		xi+=2;
	}
}

void decrypt(char *block, int nb, void *key)
{
	unsigned long *xi, xo[2], xa[2];
	unsigned long *xl, *xr;
	int i;
	char *tmp;
	BLOWFISH_CTX *ctx=(BLOWFISH_CTX *)key;

	xi=(long *)block;
	tmp=block;
	xo[0]=*xi;
	xo[1]=*(xi+1);
	xl=(unsigned long *)tmp;
	xr=(unsigned long *)(tmp+4);
	blowfish_decrypt (ctx, xl, xr);
	tmp+=8;
	xi=(long *)tmp;

	for (i=1;i<nb/8;i++) {
		xa[0]=*xi;
		xa[1]=*(xi+1);
		xl=(unsigned long *)tmp;
		xr=(unsigned long *)(tmp+4);
		blowfish_decrypt (ctx, xl, xr);
		*(xi)^=xo[0];
		*(xi+1)^=xo[1];
		xo[0]=xa[0];
		xo[1]=xa[1];
		tmp+=8;
		xi+=2;
	}
}

static void delete_key(void *key)
{
	if (key)
		kfree (key);
}


static struct tcfs_cipher_operations tcfs_blowfish_ops = {
	init_key,
	delete_key,
	encrypt,
	decrypt
};

int
init_module (void)
{
	tcfs_register_cipher ("blowfsh", &tcfs_blowfish_ops);

	return 0;
}

void
cleanup_module (void)
{
	tcfs_unregister_cipher ("blowfsh");

	return ;
}
