#include <linux/kernel.h>
#include <linux/malloc.h>
#include <linux/version.h>
#include <linux/module.h>
#include <linux/tcfs/tcfsapi.h>
#include <linux/tcfs/global.h>
#include <linux/tcfs/md5.h>

#define UINTSIZE	(sizeof(unsigned int))
#define BUFSIZE		(KEYSIZE+UINTSIZE)

static void *init_key (char *key)
{
	char *res;

	res=kmalloc(KEYSIZE, GFP_KERNEL);
	memcpy (res, key, KEYSIZE);

	return (void *)res;
}

static void encrypt(char *block, int bytes, void *key)
{
	unsigned int i=0;
	MD5_CTX context;
	unsigned char digest[TCFS_HASH_SIZE];
	char buffer[BUFSIZE];
	unsigned int *data=(unsigned int *)block;
	unsigned int *hash;

	memcpy (buffer, key, KEYSIZE);

	for (i=0;i<bytes;i+=UINTSIZE) {
		memcpy (buffer+KEYSIZE, (char *)&i, sizeof(unsigned int));
		MD5Init (&context);
		MD5Update (&context, buffer, BUFSIZE);
		MD5Final (digest, &context);
		hash=(unsigned int *)digest;
		*data ^= *hash;
		(char *)data += UINTSIZE;
	}
/* Controllare che bytes sia multiplto di 4 byte!!! */
}

static void delete_key(void *key)
{
	if (key)
		kfree (key);
}


static struct tcfs_cipher_operations tcfs_key_ops = {
	init_key,
	delete_key,
	encrypt,
	encrypt
};

int
init_module (void)
{
	tcfs_register_cipher ("key", &tcfs_key_ops);

	return 0;
}

void
cleanup_module (void)
{
	tcfs_unregister_cipher ("key");

	return ;
}
