/*
 *      $Source: /home/fergia/CVS/tcfs-linux/tcfsutils-2.0.0/src/tcfsrmkey.c,v $
 *      $Revision: 1.1.1.1 $
 *      $Date: 2000/04/07 11:21:27 $
 *      $State: Exp $
 *      $Author: fergia $
 *      $Lockers$
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <grp.h>
#include <sys/types.h>
#include <tcfslib.h>
#include "tcfsutils.h"


int main (int argc, char *argv[])
{
	const char* optstring=REMOVEKEY_OPTSTRING;
	const char* usage=REMOVEKEY_USAGE;
	char *cipher=malloc (TCFS_ENGINE_FIELD_SIZE);
	int val, result, part, threshold;
	gid_t gid;	
	uid_t uid;
	char group_name[MAX];
	option_flags flag;
	char *user;
	tcfsgpwdb* group_entry;

	RESET_WORD(flag);

	printf ("\nTCFS Utilities: version %d.%d.%d\n", MAJOR, MINOR, RELEASE);
	while ((val=getopt (argc,argv,optstring))!=-1) {
		switch (val)
		{
			case CIPHER:
				strcpy(cipher, optarg);
				SET_ISCIPHER(flag);
				break;
			case GID:
				gid=(gid_t)atoi(optarg);
				SET_ISGID(flag);
				break;
			case GROUP_NAME:
				strcpy(group_name,optarg);
				SET_ISGROUPNAME(flag);
				break;
			case REMOVEALL:
				SET_ISREMOVEALL(flag);
				break;
			case HELP:
				printf(usage,argv[0],GID,GROUP_NAME,REMOVEALL,VERBOSE,HELP);
                                exit(0);
			case VERBOSE:
				SET_ISVERBOSE(flag);
				break;
			default:
				 printf("Try '%s -%c' for more information.\n\n",argv[0],HELP);
                                exit(-1);
		}
	}

	if (optind!=argc) {
                printf("Invalid option -- %s\n",argv[optind]);
                printf("Try '%s -%c' for more information.\n\n",argv[0],HELP); 
                exit(-1);
        }     

	/* Check the options' constistency */
	if (!TEST_ISCIPHER_SET(flag)) {
		printf("You have to specify the <cipher_name>\n");
		exit (-1);
	}

	if (TEST_ISGROUPNAME_SET(flag) && TEST_ISGID_SET(flag)) {
		printf("You've specified both the gid option and the group name option.\n");
                printf("Try '%s -%c' for more information.\n\n",argv[0],HELP); 
		exit(-1);
	}
	/* End of consistency check */
	
	/* Get the group id value if group name option is set */
	if (TEST_ISGROUPNAME_SET(flag)) {
		struct group* group_info;
	
		if ((group_info=getgrnam(group_name))==NULL) {
			printf("Group with group name <%s> doesn't exist.\n",group_name);

			if (TEST_ISVERBOSE_SET(flag))
				printf("Contact your system's administrator to create this group.\n");
			printf("\n");
			exit(-1);
		}
		gid=group_info->gr_gid;
	}

	user=getlogin();
	uid=getuid();

#ifdef TCFS_DEBUG
	fprintf(stderr,"%s: user: %s uid: %d.\n",argv[0],getlogin(),getuid());
#endif


	if (TEST_ISVERBOSE_SET(flag)) { 
		if (TEST_ISREMOVEALL_SET(flag))
			printf("Removing all keys from the kernel...\n");
		else
			printf("Decrementing the key's counter by one...\n");
	}
			

	if (TEST_ISGROUPNAME_SET(flag) || TEST_ISGID_SET(flag)) {
		if (TEST_ISREMOVEALL_SET(flag))
			result=tcfs_group_disable(uid, gid, cipher,
					 TCFS_COUNTER_ALL);
		else {
			if ((group_entry=tcfs_gpwfile_fetch(user,gid))==NULL) {
				printf("%s\n",tcfs_strerror(tcfs_errno));

				if (tcfs_errno==ER_GDBMNODATA) {
					printf("There isn't an entry for group with group id <%d> into TCFS group key's file.\n",gid);
					if (TEST_ISVERBOSE_SET(flag))
						printf("Contact your system's administartor to get an entry into TCFS group key's file.\n");
				}
				printf("\n");
				exit(-1);
			}
			tcfs_gpwdb_fetch(group_entry,TCFS_DBPARTS|TCFS_DBTHRSHO,&part,&threshold);

#ifdef TCFS_DEBUG
	fprintf(stderr,"%s: gid %d, uid %d, part %d, threshold %d.\n",argv[0],gid,uid,part,threshold);
#endif

			result=tcfs_group_disable(uid, gid, cipher, 
					TCFS_COUNTER_ONE);
		}
	}
	else 
		result=tcfs_disable(uid, cipher, 
		(TEST_ISREMOVEALL_SET(flag))?TCFS_COUNTER_ALL:TCFS_COUNTER_ONE);

	if (result!=0) {

		printf("%s\n",tcfs_strerror(tcfs_errno));

		if (TEST_ISVERBOSE_SET(flag)) 
                        printf("Make sure that there is at least a tcfs filesystem mounted.\n");
                
		printf("\n");
                exit(-1);
	}

	if (TEST_ISREMOVEALL_SET(flag))
		printf("Removing completed.\n");
	else
		printf("Decrementing completed.\n");
	
	if (TEST_ISVERBOSE_SET(flag)) 
		if (tcfs_getcount()==-1)
			printf("Key was removed.\n");

	printf("\n");
			
	return 0;
}
