/*
 *      $Source: /home/fergia/CVS/tcfs-linux/tcfsutils-2.0.0/src/tcfsviewkey.c,v $
 *      $Revision: 1.1.1.1 $
 *      $Date: 2000/04/07 11:21:27 $
 *      $State: Exp $
 *      $Author: fergia $
 *      $Lockers$
 */

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <grp.h>
#include <sys/types.h> 
#include <tcfslib.h>
#include "tcfsutils.h"
#include <string.h>


int main (int argc, char *argv[]) {

	const char* optstring=VIEW_OPTSTRING;
	const char* usage=VIEW_USAGE;
	int val, result, counter=1, part, threshold ;
	gid_t gid;	
	char group_name[MAX];
	option_flags flag;
	unsigned char *key=malloc(UUKEYSIZE), *clear_key, *encoded;
	char *user, *password;
	tcfspwdb* user_entry;
	tcfsgpwdb* group_entry;

	RESET_WORD(flag);
 	printf ("\nTCFS Utilities: version %d.%d.%d\n", MAJOR, MINOR, RELEASE);
	while ((val=getopt (argc,argv,optstring))!=-1) {

		switch (val)
		{
			case GID:
				gid=(gid_t)atoi(optarg);
				SET_ISGID(flag);
				break;
			case GROUP_NAME:
				strcpy(group_name,optarg);
				SET_ISGROUPNAME(flag);
				break;
			case PERMANENT:
				counter=0;
				SET_ISPERMANENT(flag);
				break;
			case COUNTER:
				SET_ISCOUNTER(flag);
				break;
			case KEY:
				counter=0;
				SET_ISKEY(flag);
				break;
			case HELP:
				printf(usage,argv[0],GID,GROUP_NAME,PERMANENT,COUNTER,KEY,VERSION,VERBOSE,HELP);
                                exit(0);
			case VERBOSE:
				SET_ISVERBOSE(flag);
				break;
			case VERSION:
				counter=0;
				SET_ISVERSION(flag);
				break;
			default:
				 printf("Try '%s -%c' for more information.\n\n",argv[0],HELP);
                                exit(-1);
		}
	}

	if (optind!=argc) {
                printf("Invalid option -- %s\n",argv[optind]);
                printf("Try '%s -%c' for more information.\n\n",argv[0],HELP); 
                exit(-1);
        }     

	/* Check the options' constistency */
	if (TEST_ISGROUPNAME_SET(flag) && TEST_ISGID_SET(flag)) {
		printf("You've specified both the gid option and the group name option.\n");
                printf("Try '%s -%c' for more information.\n\n",argv[0],HELP); 
		exit(-1);
	}
	/* End of consistency check */
	
	/* Get the group id value if group name option is set */
	if (TEST_ISGROUPNAME_SET(flag)) {
		struct group* group_info;
	
		if ((group_info=getgrnam(group_name))==NULL) {
			printf("Group with group name <%s> doesn't exist.\n",group_name);
			if (TEST_ISVERBOSE_SET(flag))
				printf("Contact your system's administrator to create group <%s>.\n",group_name);
			printf("\n");
			exit(-1);
		}
		gid=group_info->gr_gid;
	}	

	if (TEST_ISKEY_SET(flag)) {
					
		if (geteuid()!=0) {
			printf("Command %s must have root privileges.\n\n",argv[0]);
			exit(-1);
		}

		user=getlogin();	
		password=getpass("Insert your password, please: ");
	
		if (auth (user, password)==-1) {
       			memset(password,'\0',strlen(password));
                	printf ("Invalid password for user <%s>.\n",user);

			if (TEST_ISVERBOSE_SET(flag))
				printf("Be sure to insert the proper password.\n");
			
			printf("\n");
       		        exit(-1);
	       	}	

		if (TEST_ISVERBOSE_SET(flag)) 
			printf("Getting information from TCFS key's file...\n");

		if (TEST_ISGID_SET(flag) || TEST_ISGROUPNAME_SET(flag)) {	
			if ((group_entry=tcfs_gpwfile_fetch(user,gid))==NULL) {
	
				printf("%s\n",tcfs_strerror(tcfs_errno));
		
				if (tcfs_errno==ER_GDBMNODATA) {
					printf("There isn't an entry for group with group id <%d> into TCFS group key's file.\n",gid);
	
					if (TEST_ISVERBOSE_SET(flag))
						printf("Contact your system's administrator to get an entry into TCFS group key's file.\n");
				}	
		
				printf("\n");
				exit(-1);
			}	
			tcfs_gpwdb_fetch(group_entry,TCFS_DBKEY|TCFS_DBPARTS|TCFS_DBTHRSHO,key,&part,&threshold);

			if ((clear_key=tcfs_decrypt_key(key,password,TCFS_GROUP_KEY))==NULL) {
				memset(password,'\0',strlen(password));
				printf("%s\n\n",tcfs_strerror(tcfs_errno));
				exit(-1);
			}

			printf("Group entry information:\n");
			printf("\tnumber of memebers: %d\n",part);
			printf("\tthreshold: %d\n",threshold);
			encoded=tcfs_base64_encode(clear_key,GROUPKEYSIZE);
			printf("\tgroup key's piece (base64 encoded): ");
		}
		else {
			if ((user_entry=tcfs_pwfile_fetch(user))==NULL) {
	
				printf("%s\n",tcfs_strerror(tcfs_errno));
		
				if (tcfs_errno==ER_GDBMNODATA) {
					printf("You don't have an entry into TCFS key's file.\n");
	
					if (TEST_ISVERBOSE_SET(flag) && tcfs_errno==ER_GDBMNODATA)
						printf("Contact your system's administrator to get an entry into TCFS key's file.\n");
				}
			
				printf("\n");
				exit(-1);
			}	
			tcfs_pwdb_fetch(user_entry,TCFS_DBKEY,key);

			if (strlen(key)==0) {
				memset(password,'\0',strlen(password));
				printf("You don't have a key into TCFS key's file.\n\n");
				exit(-1);
			}

			if ((clear_key=tcfs_decrypt_key(key,password,TCFS_USER_KEY))==NULL) {
				memset(password,'\0',strlen(password));
				printf("%s\n\n",tcfs_strerror(tcfs_errno));
				exit(-1);
			}	
		
			encoded=tcfs_base64_encode(clear_key,KEYSIZE);
			printf("Key (base64 encoded): ");
		}

		if (encoded==NULL) {
			printf("%s\n\n",tcfs_strerror(tcfs_errno));
			exit(-1);
		}

		printf("%s\n",encoded);

#ifdef TCFS_DEBUG 
	{
	int i;
	fprintf(stderr,"%s key:  %s\n",argv[0],key);
	fprintf(stderr,"%s clear_key: ");
	for (i=0;i<GROUPKEYSIZE;i++)
		fprintf(stderr," :%X(%d)",clear_key[i],(unsigned int) clear_key[i]);
	fprintf(stderr,"\n");
	}
#endif

	}

	if (TEST_ISPERMANENT_SET(flag)) {
	
		if (TEST_ISVERBOSE_SET(flag))
			printf("Getting information about permanent flag...\n");	
		if (TEST_ISGROUPNAME_SET(flag) || TEST_ISGID_SET(flag)) 
			result=tcfs_group_getpermanent(gid);
		else
			result=tcfs_getpermanent();

		if (result==-1) {
			printf("%s\n",tcfs_strerror(tcfs_errno));
			
			if (TEST_ISVERBOSE_SET(flag))
				printf("Make sure to pass key to the kernel first.\n");
			printf("\n");
			exit(-1);
		}
		else {
			printf("Permanent flag is ");
			if (result==0) printf("un");
			printf("set.\n");
		}
	}
	
	if (TEST_ISCOUNTER_SET(flag) || counter==1) {
	
		if (TEST_ISVERBOSE_SET(flag))
			printf("Getting the counter's value...\n");	

		if (TEST_ISGROUPNAME_SET(flag) || TEST_ISGID_SET(flag)) 
			result=tcfs_group_getcount(gid);
		else
			result=tcfs_getcount();

		if (result==-1) {
			printf("%s\n",tcfs_strerror(tcfs_errno));
			
			if (TEST_ISVERBOSE_SET(flag))
				printf("Make sure to pass key to the kernel first.\n");
			printf("\n");
			exit(-1);
		}
		else 
			printf("TCFS key counter: %d\n",result);
	}	
		
	if (TEST_ISVERSION_SET(flag)) {
		short version, minor;
		char *cipher;
	
		if (TEST_ISVERBOSE_SET(flag)) 
			printf("Getting TCFS version...\n");
	
		if (tcfs_getversion(&version,&minor,&cipher)!=0) {
			printf("%s\n",tcfs_strerror(tcfs_errno));

			if (TEST_ISVERBOSE_SET(flag))
				printf("Make sure that there is at least a tcfs filesystem mounted.\n");
			printf("\n");
			exit(-1);
		}
	
		printf("TCFS core: version %d.%d\nCipher used to encrypt/decrypt file: %s\n",version,minor,cipher);
	}

	printf("\n");	
	return 0;
}
